------------------------------------------------------------------------------
--                                                                          --
--                        ADACOM BINDING COMPONENTS                         --
--                                                                          --
--                          A D A C O M . B S T R                           --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: 1.1.1.1 $                              --
--                                                                          --
--                   Copyright (C) 1999 David Botton                        --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. It is distributed in the hope that it will be useful, but WITHOUT  --
-- ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY or  --
-- FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General Public License  --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- ADACOM is maintained by David Botton (http://www.botton.com/ada).        --
--                                                                          --
------------------------------------------------------------------------------

with Win32.Oleauto; use Win32.Oleauto;

with Interfaces.C; use Interfaces.C;
with Interfaces.C.Pointers;

with Ada.Exceptions;
with Ada.Characters.Handling;
with Ada.Unchecked_Conversion;

with System;

package body AdaCom.BSTR is

   type PWChar_t is access all Interfaces.C.WChar_t;
   -- Pointer to a WChar_t

   -------------------------------
   -- Type conversion functions --
   -------------------------------

   function To_PCWSTR is
      new Ada.Unchecked_Conversion (System.Address, Win32.PCWSTR);
   function To_PWChar_t is
      new Ada.Unchecked_Conversion (System.Address, PWChar_t);


   package WChar_Array_Pointer is new Interfaces.C.Pointers
         (index              => Interfaces.C.Size_t,
          element            => Interfaces.C.WChar_t,
          element_array      => Interfaces.C.WChar_array,
          default_terminator => Interfaces.C.Wide_Nul);

   -------------
   -- To_BSTR --
   -------------

   function To_BSTR (From_Ada : Wide_String) return Win32.OleAuto.BSTR is
   begin
      return To_BSTR(Interfaces.C.To_C(From_Ada));
   end To_BSTR;

   function To_BSTR (From_Ada : String) return Win32.OleAuto.BSTR is
   begin
      return To_BSTR (Interfaces.C.To_C(
        Ada.Characters.Handling.To_Wide_String(From_Ada)));
   end To_BSTR;

   function To_BSTR (From_C : Interfaces.C.WChar_Array)
     return Win32.OleAuto.BSTR
   is
      New_BSTR : Win32.OleAuto.BSTR;
   begin
      New_BSTR := SysAllocString( To_PCWSTR( From_C'Address ) );

      if New_BSTR = null then
         Ada.Exceptions.Raise_Exception(BSTR_Error'Identity,
           "BSTR - Unable to create BSTR");
      end if;

      return New_BSTR;
   end To_BSTR;

   function To_BSTR (From_C : Interfaces.C.Char_Array)
      return Win32.OleAuto.BSTR
   is
   begin
      return To_BSTR (Interfaces.C.To_C(
        Ada.Characters.Handling.To_Wide_String(
                      Interfaces.C.To_Ada(From_C))));
   end To_BSTR;

   ------------
   -- To_Ada --
   ------------

   function To_Ada (From_BSTR : Win32.OleAuto.BSTR) return Wide_String is
   begin
      return Interfaces.C.To_Ada(To_C(From_BSTR));
   exception
      when E : NULL_BSTR =>
         return "";
   end To_Ada;

   function To_Ada (From_BSTR : Win32.OleAuto.BSTR) return String is
   begin
      return Ada.Characters.Handling.To_String(To_Ada (From_BSTR));
   exception
      when E : NULL_BSTR =>
         return "";
   end To_Ada;

   ----------
   -- To_C --
   ----------

   function To_C (From_BSTR : Win32.OleAuto.BSTR)
     return Interfaces.C.WChar_Array
   is
   begin
      if From_BSTR = null then
         Ada.Exceptions.Raise_Exception(NULL_BSTR'Identity,"BSTR is null");
      end if;

      return WChar_Array_Pointer.value (WChar_Array_Pointer.pointer(
        To_PWChar_t(From_BSTR.all'Address)));
   end To_C;

   function To_C (From_BSTR : Win32.OleAuto.BSTR)
      return Interfaces.C.Char_Array
   is
   begin
      return Interfaces.C.To_C(Ada.Characters.Handling.To_String(
        Interfaces.C.To_Ada(To_C(From_BSTR))));
   end To_C;

   ----------
   -- Free --
   ----------

   procedure Free (This : Win32.OleAuto.BSTR) is
   begin
      SysFreeString(This);
   end Free;

end AdaCom.BSTR;