------------------------------------------------------------------------------
--                                                                          --
--                        ADACOM BINDING COMPONENTS                         --
--                                                                          --
--                          A D A C O M . B S T R                           --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                            $Revision: 1.1.1.1 $                              --
--                                                                          --
--                   Copyright (C) 1999 David Botton                        --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. It is distributed in the hope that it will be useful, but WITHOUT  --
-- ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY or  --
-- FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General Public License  --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- ADACOM is maintained by David Botton (http://www.botton.com/ada).        --
--                                                                          --
------------------------------------------------------------------------------

-- BSTR conversion functions. BSTRs are used by COM to pass strings. They are
-- wchar_t*'s under the hood with their size prepended to the pointer.
-- These conversion routines create and assume null terminated BSTRs.

with Win32.OleAuto;
with Interfaces.C;

package AdaCom.BSTR is

   function To_BSTR( From_Ada : Wide_String ) return Win32.OleAuto.BSTR;
   function To_BSTR( From_Ada : String ) return Win32.OleAuto.BSTR;
   function To_BSTR( From_C : Interfaces.C.WChar_Array )
     return Win32.OleAuto.BSTR;
   function To_BSTR( From_C : Interfaces.C.Char_Array )
     return Win32.OleAuto.BSTR;
   -- These conversion functions create a new BSTR that requires a Free
   -- to destroy


   function To_Ada( From_BSTR : Win32.OleAuto.BSTR ) return Wide_String;
   function To_Ada( From_BSTR : Win32.OleAuto.BSTR ) return String;
   function To_C( From_BSTR : Win32.OleAuto.BSTR )
     return Interfaces.C.WChar_Array;
   function To_C( From_BSTR : Win32.OleAuto.BSTR )
     return Interfaces.C.Char_Array;
   -- These conversion functions do not destroy the original BSTR
   -- The BSTR to be converted must not be NULL when converting To_C

   procedure Free( This : Win32.OleAuto.BSTR );
   -- This conversion can free BSTRs created with this packages conversion
   -- functions and those create with the Win32 API function SysAllocString

   NULL_BSTR, BSTR_Error : exception;

end AdaCom.BSTR;