------------------------------------------------------------------------------
--                                                                          --
--                        ADACOM BINDING COMPONENTS                         --
--                                                                          --
--                               A D A C O M                                --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: 1.1.1.1 $                          --
--                                                                          --
--                   Copyright (C) 1999 David Botton                        --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. It is distributed in the hope that it will be useful, but WITHOUT  --
-- ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY or  --
-- FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General Public License  --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- ADACOM is maintained by David Botton (http://www.botton.com/ada).        --
--                                                                          --
------------------------------------------------------------------------------

-- AdaCom is a collection of packages for use in interfacing Ada with Type
-- Libraries, COM/DCOM Objects, and OLE

with Ada.Unchecked_Conversion;
with Ada.Characters.Handling;
with Interfaces.C.Pointers;
with Interfaces.C; use Interfaces.C;

with Stdole; use Stdole;

package body AdaCom is
   pragma Linker_Options("-lole32");
   --pragma Linker_Options("-loleaut32");

   type Pointer_To_WChar_t is access all Interfaces.C.WChar_t;

   package WChar_Array_Pointer is new Interfaces.C.Pointers
     (index              => Interfaces.C.Size_t,
      element            => Interfaces.C.WChar_t,
      element_array      => Interfaces.C.WChar_array,
      default_terminator => Interfaces.C.Wide_Nul);

   function StringFromCLSID (Rclsid : access Stdole.GUID;
                             Lplpsz : access Pointer_To_WChar_t)
                             return Stdole.HRESULT;
   pragma Import(Stdcall, StringFromCLSID, "StringFromCLSID");

   procedure CoTaskMemFree (Pv : in Pointer_To_WChar_t);
   pragma Import(Stdcall, CoTaskMemFree, "CoTaskMemFree");

   function CLSIDFromString (StringUuid : Interfaces.C.WChar_Array;
                             Uuid       : access Stdole.GUID)
                             return Stdole.HRESULT;
   pragma Import(Stdcall, CLSIDFromString, "CLSIDFromString");

   function CLSIDFromProgID (LpszProgID : Interfaces.C.WChar_Array;
                             Lpclsid    : access Stdole.GUID)
                             return Stdole.HRESULT;
   pragma Import(Stdcall, CLSIDFromProgID, "CLSIDFromProgID");

   -------------------------------
   -- Type conversion functions --
   -------------------------------

   function To_GUID is
      new Ada.Unchecked_Conversion (Win32.Rpcdce.GUID, Stdole.GUID);

   -----------------
   -- Format_GUID --
   -----------------

   function Format_GUID (guid : Win32.Rpcdce.GUID) return string is
   begin
      return To_String(To_GUID(Guid));
   end Format_GUID;

   -------------
   -- To_GUID --
   -------------

   function To_GUID(GUID : String) return Stdole.GUID is
      use Interfaces.C;

      UUID_String : WChar_Array := To_C(
       Ada.Characters.Handling.To_Wide_String(GUID));
      UUID        : aliased Stdole.GUID;
   begin
      if CLSIDFromString(UUID_String, UUID'Unchecked_Access) /= S_OK then
         raise GUID_Conversion_Error;
      end if;

      return UUID;
   end To_GUID;

   ---------------
   -- To_String --
   ---------------

   function To_String(GUID : Stdole.GUID) return String is
      use Interfaces.C;

      Ref_GUID    : aliased Stdole.GUID := GUID;
      GUID_String : aliased Pointer_To_WChar_t;
   begin
      if StringFromCLSID(Ref_Guid'Unchecked_Access, GUID_String'Unchecked_Access) = S_OK then
         declare
            Ada_GUID_String : String :=
              Ada.Characters.Handling.To_String(To_Ada(
                WChar_Array_Pointer.value (WChar_Array_Pointer.pointer(
                  GUID_String))));
         begin
            CoTaskMemFree(GUID_String);
            return Ada_GUID_String;
         end;
      else
         raise GUID_Conversion_Error;
      end if;
   end To_String;

   -----------------------
   -- CLSID_From_PROGID --
   -----------------------

   function CLSID_From_PROGID (PROGID : String) return Stdole.GUID is
      Program_ID : WChar_Array
        := To_C(Ada.Characters.Handling.To_Wide_String(PROGID));

      Class_ID : aliased Stdole.GUID;
   begin
      if CLSIDFromProgID(Program_ID, Class_ID'Unchecked_Access) /= S_OK then
         raise Invalid_PROGID_Error;
      end if;

      return Class_ID;
   end CLSID_From_PROGID;


end AdaCom;
