with Win32.WinUser;
with Win32.Winerror; use Win32.Winerror;
with Win32.Rpcdce; use Win32.Rpcdce;
with Win32.Winbase;
with System; use System;

with Ada.Unchecked_Deallocation;
with Ada.Unchecked_Conversion;

with Beep_Global; use Beep_Global;

with Adacom.BSTR;

package body BeepCom is

   ----------------------
   -- Type Conversions --
   ----------------------

   procedure Free is
     new Ada.Unchecked_Deallocation(Interface, Interface_Pointer);
   procedure Free is
     new Ada.Unchecked_Deallocation(BeepClass, BeepClass_Pointer);
   function To_Interface_Pointer is
     new Ada.Unchecked_Conversion (System.Address, Interface_Pointer);
   function To_PVOID is
     new Ada.Unchecked_Conversion (System.Address, Win32.PVOID);
   function To_LPCSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.LPCSTR);

   -----------------
   -- Message_Box --
   -----------------

   procedure Message_Box ( Title, Message : String) is
      BoxTitle    : Char_Array := To_C(Title);
      BoxMessage  : Char_Array := To_C(Message);
      RetValue : Win32.Int;
   begin
      RetValue:=Win32.WinUser.MessageBox(System.Null_Address,
                                         To_LPCSTR(BoxMessage'Address),
                                         To_LPCSTR(BoxTitle'Address),
                                         Win32.WinUser.MB_YESNO);
   end Message_Box;

   ----------
   -- Free --
   ----------

   procedure Free ( Pointer : in System.Address ) is
      X       : Interface_Pointer;
      lResult : Win32.Long;
   begin
      X := To_Interface_Pointer(Pointer);

      if X.CoClass.Ref_Count < 1 then
         -- All interfaces to this COM object have been released
         -- so free the COM objects data completing the release
         -- of the COM object from memory
         Free(X.CoClass);
         lResult := Win32.Winbase.InterlockedDecrement(g_cComponents'Access);
      end if;

      Free(X);
      Can_Close;
   end Free;

   --------------------
   -- QueryInterface --
   --------------------

   function QueryInterface  (This      : access Interface;
                             riid      : REFIID;
                             ppvObject : access Win32.PVOID)
                             return HRESULT
   is
      New_Interface : aliased Interface_Pointer;
      lResult : Win32.Long;
   begin

      if riid.all = Beep_Global.IID_IUnknown then
         -- Since IUnknown is the parent of every interface, just
         -- return back a pointer to this interface with an additional
         -- reference count and the client will use it as if it was
         -- IUnknown
         ppvObject.all := This.all'Address;
         lResult := Win32.Winbase.InterlockedIncrement(This.Ref_Count'Access);
      elsif riid.all = Beep_Global.IID_IBeep then
         New_Interface := new Interface;
         New_Interface.CoClass := This.CoClass;
         New_Interface.Vtbl := Beep_Vtbl'Address;
         ppvObject.all := New_Interface.all'Address;
      elsif Riid.all = Beep_Global.IID_IMessage then
         New_Interface := new Interface;
         New_Interface.CoClass := This.CoClass;
         New_Interface.Vtbl := Message_Vtbl'Address;
         ppvObject.all := New_Interface.all'Address;
      elsif Riid.all = Beep_Global.IID_IStat then
         New_Interface := new Interface;
         New_Interface.CoClass := This.CoClass;
         New_Interface.Vtbl := Stat_Vtbl'Address;
         ppvObject.all := New_Interface.all'Address;
      else
         ppvObject.all := System.Null_Address;
         return E_NOINTERFACE;
      end if;

      return S_OK;
   end QueryInterface;

   ------------
   -- AddRef --
   ------------

   function AddRef (This : access Interface)
                    return Win32.ULONG
   is
      lResult : Win32.Long;
   begin
      -- InterlockedIncrement is a thread protected Win32 API function
      -- increment a long

      -- Interface reference increment
      lResult := Win32.Winbase.InterlockedIncrement(This.Ref_Count'Access);

      -- Object reference increment
      lResult :=
        Win32.Winbase.InterlockedIncrement(This.CoClass.Ref_Count'Access);

      return Win32.ULONG(This.Ref_Count);
   end AddRef;

   -------------
   -- Release --
   -------------

   function Release (This : access Interface)
                     return Win32.ULONG
   is
      lResult : Win32.Long;
   begin
      -- InterlockedDecrement is a thread protected Win32 API function
      -- decrement a long
      lResult :=
        Win32.Winbase.InterlockedDecrement(This.CoClass.Ref_Count'Access);

      if Win32.Winbase.InterlockedDecrement(This.Ref_Count'Access) /=0 then
         -- This.Ref_Count should never be less then 0, so if it is
         -- we get a CONSTRAINT_ERROR exception from Ada
         return Win32.ULONG(This.Ref_Count);
      else
         -- Last reference to IClassFactory and IUnknown released,
         -- so free the object
         Free(This.all'Address);
         return 0;
      end if;

   end Release;

   ----------
   -- Beep --
   ----------

   function Beep (This : access Interface)
                  return HRESULT
   is
      RetValue    : Win32.Bool;
   begin
      RetValue := Win32.WinUser.MessageBeep(Win32.WinUser.MB_ICONEXCLAMATION);
      Message_Box("Beep", "Did you hear it?");
      This.CoClass.Beeps := This.CoClass.Beeps + 1;
      return S_OK;
   end Beep;

   -------------
   -- Display --
   -------------

   function Display (This    : access Interface;
                     Message : Win32.OleAuto.BSTR)
                     return HRESULT
   is
   begin
      Message_Box("Beep", Adacom.BSTR.To_Ada(Message));
      This.CoClass.Beeps := This.CoClass.Beeps + 1;
      return S_OK;
   end Display;

   -----------
   -- Beeps --
   -----------

   function Beeps (This  : access Interface;
                   Calls : access Interfaces.C.Int)
                   return HRESULT
   is
   begin
      Calls.all := Interfaces.C.int(This.CoClass.Beeps);
      return S_OK;
   end Beeps;


end BeepCom;









