with Interfaces.C; use Interfaces.C;
with System; use System;

with Win32.Objbase; use Win32.Objbase;
with Win32.Oleauto;

package BeepCom is
   subtype HRESULT is Unsigned_Long;

   type Interface;
   type Interface_Pointer is access all Interface;
   -- Generic Interface Object

   -- Access function types for IUnknown part of all interfaces

   type Af_QueryInterface is access function (This      : access Interface;
                                              riid      : REFIID;
                                              ppvObject : access Win32.PVOID)
     return HRESULT;
   pragma Convention(Stdcall, Af_QueryInterface);

   type Af_AddRef is access function (This : access Interface)
     return Win32.ULONG;
   pragma Convention(Stdcall, Af_AddRef);

   type Af_Release is access function (This : access Interface)
     return Win32.ULONG;
   pragma Convention(Stdcall, Af_Release);

   -- Access function types for IBeep part of all interfaces

   type Af_Beep is access function (This : access Interface)
     return HRESULT;
   pragma Convention(Stdcall, Af_Beep);

   -- Access function types for IMessage part of all interfaces

   type Af_Display is access function (This : access Interface;
                                       Message : Win32.OleAuto.BSTR)
     return HRESULT;
   pragma Convention(Stdcall, Af_Display);

   -- Access function types for IStat part of all interfaces

   type Af_Beeps is access function (This  : access Interface;
                                     Calls : access Interfaces.C.int)
     return HRESULT;
   pragma Convention(Stdcall, Af_Beeps);

   -- Function specs for IUnknown
   --
   -- The QueryInterface, Addref and Release are the first three functions
   -- in every interface since every interface is a child of IUnknown. This
   -- allows every interface to be treated polymorphically as
   -- IUnknown. Then QueryInterface can be used to query the object for
   -- support of an interface and retrieve a pointer to the COM object's
   -- interface that conforms to the requested interface.  In essence
   -- QueryInterface is used to do a typesafe "cast" from one interface of
   -- a COM object to another.

   function QueryInterface  (This      : access Interface;
                             riid      : REFIID;
                             ppvObject : access Win32.PVOID)
                             return HRESULT;
   pragma Convention(Stdcall, QueryInterface);
   -- Query COM object for the interface riid and return a pointer to it
   -- if possible. QueryInterface will call AddRef on the interface before
   -- returning it in ppvObject

   function AddRef (This : access Interface)
                    return Win32.ULONG;
   pragma Convention(Stdcall, AddRef);
   -- Add a reference count to the object

   function Release (This : access Interface)
                     return Win32.ULONG;
   pragma Convention(Stdcall, Release);
   -- Reduce reference count. If 0 free object from memory

   -- Function specs for IBeep

   function Beep (This : access Interface)
                  return HRESULT;
   pragma Convention(Stdcall, Beep);
   -- Beeps and pops a dialog box

   -- Function specs for IMessage

   function Display (This    : access Interface;
                     Message : Win32.OleAuto.BSTR)
                     return HRESULT;
   pragma Convention(Stdcall, Display);
   -- Pops a dialog box with Message in it

   -- Function specs for IStat

   function Beeps (This  : access Interface;
                   Calls : access Interfaces.C.Int)
                   return HRESULT;
   pragma Convention(Stdcall, Beeps);
   -- Returns how many times dialog boxes have been popped.


   -- Interface definitions

   type Beep_Vtbl_Record is
      record
         QueryInterface : Af_QueryInterface := BeepCom.QueryInterface'access;
         AddRef         : Af_AddRef := BeepCom.AddRef'access;
         Release        : Af_Release := BeepCom.Release'access;
         Beep           : Af_Beep := BeepCom.Beep'access;
      end record;
   pragma Convention(C, Beep_Vtbl_Record);
   type Beep_Vtbl_Pointer is access all Beep_Vtbl_Record;
   -- IBeep table of functions

   type Message_Vtbl_Record is
      record
         QueryInterface : Af_QueryInterface := BeepCom.QueryInterface'access;
         AddRef         : Af_AddRef := BeepCom.AddRef'access;
         Release        : Af_Release := BeepCom.Release'access;
         Display        : Af_Display := BeepCom.Display'access;
      end record;
   pragma Convention(C, Message_Vtbl_Record);
   type Message_Vtbl_Pointer is access all Message_Vtbl_Record;
   -- IMessage table of functions

   type Stat_Vtbl_Record is
      record
         QueryInterface : Af_QueryInterface := BeepCom.QueryInterface'access;
         AddRef         : Af_AddRef := BeepCom.AddRef'access;
         Release        : Af_Release := BeepCom.Release'access;
         Beeps          : Af_Beeps := BeepCom.Beeps'access;
      end record;
   pragma Convention(C, Stat_Vtbl_Record);
   type Stat_Vtbl_Pointer is access all Stat_Vtbl_Record;
   -- IStat table of functions

   Beep_Vtbl : aliased Beep_Vtbl_Record;
   Message_Vtbl : aliased Message_Vtbl_Record;
   Stat_Vtbl : aliased Stat_Vtbl_Record;
   -- Create image of function tables in memory

   type BeepClass is
      record
         Ref_Count : aliased Win32.LONG := 1;
         Beeps     : Integer := 0;
      end record;
   type BeepClass_Pointer is access all BeepClass;
   -- Internal data for for the BeepClass COM object. Notice that there is
   -- no Convention pragma on this type. This type is an Ada type.

   type Interface is
      record
         Vtbl      : System.Address := Beep_Vtbl'Address;
         Ref_Count : aliased Win32.LONG := 1;
         CoClass   : BeepClass_Pointer;
      end record;
   pragma Convention(C, Interface);
   -- The Interface is contstructed so that the first element in the record
   -- is a pointer to the table of functions and additional members are
   -- private data elements of the interface unavailable to clients of the
   -- COM object. The CoClass element holds a pointer to the object wide
   -- data elements of the COM object.
end BeepCom;
