with win32.objbase; use win32.objbase;
with win32.winerror; use Win32.winerror;
with win32.rpcdce; use win32.rpcdce;
with win32.winbase;
with win32.oleauto;
with Win32.Windef;
with Win32.Winreg;
with Win32.Winnt;

with interfaces.c; use interfaces.c;
with ada.characters.handling;
with Ada.Unchecked_Conversion;
with system;

with Beep_Global; use Beep_Global;
with Factory; use Factory;

package body BeepDll is
   -------------
   -- Imports --
   -------------

   procedure Adainit;
   pragma import (C, Adainit);

   ----------------------
   -- Type Conversions --
   ----------------------

   function To_PCBYTE is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCBYTE);
   function To_PCCH is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCCH);
   function To_PCWSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCWSTR);
   function To_PWSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PWSTR);
   function To_LPSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.LPSTR);
   function To_LPCSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.LPCSTR);
   function To_PCHAR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCHAR);

   -----------------
   -- Definitions --
   -----------------

   CLSID               : constant String :=
     "{0FE0EE21-8AA2-11d2-81AA-444553540001}";
   Library_Name        : constant String := "BeepLibrary.BeepClass.1";
   Library_Version     : constant String :="1";
   Library_Description : constant String :="Beep Class";

   -------------
   -- DllMain --
   -------------

   function DllMain (hinstDLL    : Win32.Windef.HINSTANCE;
                     fdwReason   : Win32.DWORD;
                     lpvReserved : Win32.LPVOID)
     return Win32.BOOL;
   pragma Export (StdCall, DllMain, "DllMain");

   function DllMain (hinstDLL    : Win32.Windef.HINSTANCE;
                     fdwReason   : Win32.DWORD;
                     lpvReserved : Win32.LPVOID)
     return Win32.BOOL
   is
   begin
      if FdwReason = 1 then
         AdaInit;
      end if;

      -- Save instance handle for later use.
      hInstance := hinstDll;
      return 1;
   end DllMain;

   -----------------------
   -- DllGetClassObject --
   -----------------------

   function DllGetClassObject(clsid : REFCLSID;
                              riid  : REFIID;
                              ppv   : access Win32.PVOID)
     return Unsigned_Long
   is
      pFactory : IClassFactory_Pointer;
      hr       : unsigned_long := 16#80040111#;
      refcount : Win32.ULONG;
   begin
      -- A server (DLL or EXE) can host many different COM Objects

      -- This server only handles one type
      if clsid.all /= CLSID_BeepClass then
         return CLASS_E_CLASSNOTAVAILABLE;
      end if;

      -- The Class Factory used to create the COM object that has CLSID
      -- is created and returned.

      pFactory := new Factory.IClassFactory;

      hr := pFactory.vtbl.QueryInterface(pFactory, riid, ppv);

      refcount := pFactory.vtbl.Release(pFactory);

      return hr;
   end DllGetClassObject;

   ---------------------
   -- DllCanUnloadNow --
   ---------------------

   function DllCanUnloadNow return unsigned_long is
   begin
      if (g_cServer=0) and (g_cComponents=0) then
         return S_OK;
      else
         return S_FALSE;
      end if;
   end DllCanUnloadNow;

   --------------
   -- Register --
   --------------

   procedure Register(KeyName, ValueName, Value : String) is
      hkey       : aliased Win32.Winreg.HKEY;
      err        : Win32.Long;
      cKeyName   : aliased char_array := To_C(KeyName);
      cValueName : aliased char_array := To_C(ValueName);
      cValue     : aliased char_array := To_C(Value);
   begin
      err := Win32.Winreg.RegCreateKey(Win32.Winreg.HKEY_CLASSES_ROOT,
                                       To_PCCH(cKeyName'Address),
                                       hkey'Unchecked_Access);

      err := Win32.Winreg.RegSetValueEx(hkey, To_PCCH(cValueName'Address),
                                        0,
                                        Win32.Winnt.REG_SZ,
                                        To_PCBYTE(cValue'Address),
                                        cValue'Length);

      err := Win32.Winreg.RegCloseKey(hkey);
   end Register;

   ----------------
   -- Unregister --
   ----------------

   procedure Unregister(KeyName : String) is
      err        : Win32.Long;
      cKeyName   : aliased char_array := To_C(KeyName);
   begin
      err := Win32.Winreg.RegDeleteKey(Win32.Winreg.HKEY_CLASSES_ROOT,
                                       To_PCCH(cKeyName'Address));
   end Unregister;

   -----------------------
   -- DllRegisterServer --
   -----------------------

   function DllRegisterServer return unsigned_long is
      hr        : Win32.Objbase.HRESULT;
      dr        : Win32.DWORD;
      MAX_PATH  : constant := 1024;
      DllPath   : aliased char_array(1..MAX_PATH);
      pTypeLib  : aliased Win32.OleAuto.LPTYPELIB;
      refcount  : Win32.Ulong;
   begin
      dr:=Win32.Winbase.GetModuleFileName(hInstance,
                                          To_PCHAR(DllPath'Address),
                                          MAX_PATH);
      declare
         wFilePath : aliased wchar_array := To_C(
            ada.characters.handling.to_wide_string (To_Ada(DLLPath)));
      begin
         hr:=Win32.OleAuto.LoadTypeLib(To_PCWSTR(wFilePath'Address),
                                       pTypeLib'Unchecked_Access);
         hr:=Win32.OleAuto.RegisterTypeLib(pTypeLib,
                                           To_PWSTR(wFilePath'Address),
                                           To_PWSTR(System.Null_Address));
         refcount := pTypeLib.lpVtbl.Release(pTypeLib);
      end;

      Register("CLSID\" & CLSID, "", "Beep Class" );
      Register("CLSID\" & CLSID, "AppID", CLSID );
      Register("CLSID\" & CLSID & "\InProcServer32","", To_Ada(DllPath) );
      Register("CLSID\" & CLSID & "\InProcServer32",
               "ThreadingModel",
               "Apartment");
      Register("CLSID\" & CLSID & "\ProgID",
               "",
               Library_Name & "." & Library_Version);
      Register("CLSID\" & CLSID & "\VersionIndependentProgID",
               "",
               Library_Name);
      Register(Library_Name, "", Library_Description);
      Register(Library_Name & "\CLSID", "", CLSID );
      Register(Library_Name & "\CurVer",
               "",
               Library_Name & "." & Library_Version);
      Register(Library_Name & "." & Library_Version,
               "",
               Library_Name);
      Register(Library_Name & "." & Library_Version & "\CLSID",
               "",
               CLSID );
      Register("AppID\" & CLSID, "", Library_Description);
      Register("AppID\" & CLSID, "DllSurrogate", "" );

      return S_OK;
   end DllRegisterServer;

   -----------------------
   -- UnregisterTypeLib --
   -----------------------

   function UnregisterTypeLib(libid     : access Win32.Objbase.IID;
                              wVerMajor : Win32.WORD;
                              wVerMinor : Win32.WORD;
                              lcid      : Win32.Winnt.LCID;
                              syskind   : Win32.Oleauto.SYSKIND)
                              return Win32.Objbase.HRESULT;
   pragma Import(Stdcall, UnRegisterTypeLib, "UnRegisterTypeLib");


   -------------------------
   -- DllUnregisterServer --
   -------------------------

   function DllUnregisterServer return unsigned_long is
      hr : Win32.Objbase.HRESULT;
   begin
      hr := UnregisterTypeLib(LIBID_BeepLibrary'Unchecked_Access,
                              1,
                              0,
                              Win32.Winnt.LANG_NEUTRAL,
                              Win32.OleAuto.SYS_WIN32);

      Unregister("CLSID\" & CLSID & "\InProcServer32");
      Unregister("CLSID\" & CLSID & "\ProgID");
      Unregister("CLSID\" & CLSID & "\VersionIndependentProgID");
      Unregister("CLSID\" & CLSID);
      Unregister(Library_Name & "\CLSID");
      Unregister(Library_Name & "\CurVer");
      Unregister(Library_Name);
      Unregister(Library_Name & "." & Library_Version & "\CLSID");
      Unregister(Library_Name & "." & Library_Version);
      Unregister("AppID\" & CLSID);
      return S_OK;
   end DllUnregisterServer;

end BeepDll;





























