with Win32.objbase; use Win32.objbase;
with Win32.winerror; use Win32.winerror;
with Win32.oleauto;
with win32.winnt;
with win32.winbase;
with Win32.Winuser;
with Win32.Winreg;
with Win32.WinMain;
with Win32;

with Ada.Text_IO; use Ada.Text_IO;
with interfaces.c; use interfaces.c;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Unchecked_Conversion;
with Ada.Characters.Handling;
with System;

with Beep_Global; use Beep_Global;
with Factory; use Factory;

procedure BeepExe is
   use type Win32.BOOL;

   --------------------
   -- Linker Options --
   --------------------

   pragma Linker_Options ("beeprc.rbj");
   pragma Linker_Options ("oleaut32.lib");
   pragma Linker_Options ("-lole32");
   pragma Linker_Options ("-luser32");

   ----------------------
   -- Type Conversions --
   ----------------------

   function To_LPUNKNOWN is
     new Ada.Unchecked_Conversion (IClassFactory_Pointer, LPUNKNOWN);
   function To_PCBYTE is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCBYTE);
   function To_PCCH is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCCH);
   function To_PCWSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCWSTR);
   function To_PWSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PWSTR);
   function To_LPSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.LPSTR);
   function To_LPCSTR is
     new Ada.Unchecked_Conversion (System.Address, Win32.LPCSTR);
   function To_PCHAR is
     new Ada.Unchecked_Conversion (System.Address, Win32.PCHAR);

   -----------------
   -- Definitions --
   -----------------

   CLSID               : constant String :=
     "{0FE0EE21-8AA2-11d2-81AA-444553540001}";
   Library_Name        : constant String := "BeepLibrary.BeepClass.1";
   Library_Version     : constant String :="1";
   Library_Description : constant String :="Beep Class";

   --------------
   -- Register --
   --------------

   procedure Register(KeyName, ValueName, Value : String) is
      hkey       : aliased Win32.Winreg.HKEY;
      err        : Win32.Long;
      cKeyName   : aliased char_array := To_C(KeyName);
      cValueName : aliased char_array := To_C(ValueName);
      cValue     : aliased char_array := To_C(Value);
   begin
      err := Win32.Winreg.RegCreateKey(Win32.Winreg.HKEY_CLASSES_ROOT,
                                       To_PCCH(cKeyName'Address),
                                       hkey'Unchecked_Access);
      err := Win32.Winreg.RegSetValueEx(hkey, To_PCCH(cValueName'Address),
                                        0,
                                        Win32.Winnt.REG_SZ,
                                        To_PCBYTE(cValue'Address),
                                        cValue'Length);
      err := Win32.Winreg.RegCloseKey(hkey);
   end Register;

   ----------------
   -- Unregister --
   ----------------

   procedure Unregister(KeyName : String) is
      err        : Win32.Long;
      cKeyName   : aliased char_array := To_C(KeyName);
   begin
      err := Win32.Winreg.RegDeleteKey(Win32.Winreg.HKEY_CLASSES_ROOT,
                                       To_PCCH(cKeyName'Address));
   end Unregister;

   -----------------------
   -- UnregisterTypeLib --
   -----------------------

   function UnRegisterTypeLib(libid     : access Win32.Objbase.IID;
                              wVerMajor : Win32.WORD;
                              wVerMinor : Win32.WORD;
                              lcid      : Win32.Winnt.LCID;
                              syskind   : Win32.Oleauto.SYSKIND)
                              return Win32.Objbase.HRESULT;
   pragma Import(Stdcall, UnRegisterTypeLib, "UnRegisterTypeLib");

   ---------------------
   -- Local Variables --
   ---------------------

   hr                : Win32.Winerror.HRESULT;
   lResult           : Win32.LRESULT;
   pFactory          : IClassFactory_Pointer;
   dwRegister        : aliased Win32.DWORD;
   refcount          : Win32.ULONG;
   tMSG              : aliased Win32.Winuser.MSG;
   MSG               : Win32.Winuser.LPMSG := tMSG'Unchecked_Access;
   argument_error,
   com_error         : exception;

begin
   -- Check command line for RegServer/UnRegServer/Embedding

   if Argument_Count /= 1 then
      Put_Line("This is a local server for the Beep COM object");
      Put_Line("To register this server use:");
      Put_Line("beepexe -RegServer");
      New_Line;
      Put_Line("To unregister this server use:");
      Put_Line("beepexe -UnregServer");
      New_Line;
      Put_Line("To start the server up manually: (COM will do it for you)");
      Put_Line("beepexe -Embedding");

      raise Argument_Error;
   end if;

   if (Argument(1) = "/Embedding") or (Argument(1) = "-Embedding") then
      -- Will be creating a LocalServer
      InProcServer := False;

      -- Initialize Com Libraries
      hr := CoInitialize(system.null_address);
      if hr /= S_OK then
         put_line("Unable to initialize COM libraries.");
         put_line("HR = " & Win32.Winerror.HRESULT'IMAGE(hr));
         raise com_error;
      end if;

      -- Start Factory
      pFactory := new Factory.IClassFactory;

      hr := CoRegisterClassObject(
               CLSID_BeepClass'Unchecked_Access,
                 To_LPUNKNOWN(pFactory),
                   4, --CLSCTX_LOCAL_SERVER
                   1, --REGCLS_MULTIPLEUSE,
                   dwRegister'Unchecked_Access) ;

      if hr /= S_OK then
         refcount := pFactory.vtbl.Release(pFactory);
         put_line("Unable to register IClassFactory.");
         put_line("HR = " & Win32.Winerror.HRESULT'IMAGE(hr));
         raise com_error;
      end if;

      while(Win32.Winuser.GetMessage (MSG, System.Null_Address,
               Win32.UINT(0), Win32.UINT(0)) /= 0) loop
         lResult := Win32.Winuser.DispatchMessage(Win32.WinUser.ac_MSG_t(MSG));
      end loop;

      -- Stop Factory
      hr := CoRevokeClassObject(dwRegister) ;
      refcount := pFactory.vtbl.Release(pFactory);

      -- Uninitialize the COM libraries
      CoUninitialize;

   elsif (Argument(1) = "/RegServer") or (Argument(1) = "-RegServer") then
      declare
         dr        : Win32.DWORD;
         MAX_PATH  : constant := 1024;
         ExePath   : aliased char_array(1..MAX_PATH);
         pTypeLib  : aliased Win32.OleAuto.LPTYPELIB;
         refcount  : Win32.Ulong;
      begin

         dr:=Win32.Winbase.GetModuleFileName(Win32.WinMain.Get_Hinstance,
                                             To_PCHAR(ExePath'Address),
                                             MAX_PATH);
         declare
            wFilePath : aliased wchar_array := To_C
              (ada.characters.handling.to_wide_string (To_Ada(ExePath)));
         begin
            hr:=Win32.OleAuto.LoadTypeLib(To_PCWSTR(wFilePath'Address),
                                          pTypeLib'Unchecked_Access);
            hr:=Win32.OleAuto.RegisterTypeLib(pTypeLib,
                                              To_PWSTR(wFilePath'Address),
                                              To_PWSTR(System.Null_Address));
            refcount := pTypeLib.lpVtbl.Release(pTypeLib);
         end;

         Register("CLSID\" & CLSID, "", "Beep Class" );
         Register("CLSID\" & CLSID, "AppID", CLSID );
         Register("CLSID\" & CLSID & "\LocalServer32","", To_Ada(ExePath) );
         Register("CLSID\" & CLSID & "\LocalServer32",
                  "ThreadingModel",
                  "Apartment");
         Register("CLSID\" & CLSID & "\ProgID",
                  "",
                  Library_Name & "." & Library_Version);
         Register("CLSID\" & CLSID & "\VersionIndependentProgID",
                  "",
                  Library_Name);
         Register(Library_Name, "", Library_Description);
         Register(Library_Name & "\CLSID", "", CLSID );
         Register(Library_Name & "\CurVer",
                  "",
                  Library_Name & "." & Library_Version);
         Register(Library_Name & "." & Library_Version,
                  "",
                  Library_Name);
         Register(Library_Name & "." & Library_Version & "\CLSID",
                  "",
                  CLSID );
         Register("AppID\" & CLSID, "", Library_Description);

         Put_Line("Registered Beep Object Server");
      end;

   elsif (Argument(1) = "/UnregServer") or (Argument(1) = "-UnregServer") then
      hr := UnRegisterTypeLib(LIBID_BeepLibrary'Unchecked_Access, 1, 0,
               Win32.Winnt.LANG_NEUTRAL, Win32.OleAuto.SYS_WIN32);

      Unregister("CLSID\" & CLSID & "\LocalServer32");
      Unregister("CLSID\" & CLSID & "\ProgID");
      Unregister("CLSID\" & CLSID & "\VersionIndependentProgID");
      Unregister("CLSID\" & CLSID);
      Unregister(Library_Name & "\CLSID");
      Unregister(Library_Name & "\CurVer");
      Unregister(Library_Name);
      Unregister(Library_Name & "." & Library_Version & "\CLSID");
      Unregister(Library_Name & "." & Library_Version);
      Unregister("AppID\" & CLSID);

      Put_Line("Unregistered Beep Object Server");
   else
      put_line("Invalid arguments");
   end if;
end BeepExe;
