with Win32.Winerror; use Win32.Winerror;
with Win32.Rpcdce; use Win32.Rpcdce;
with Win32.Winbase;
with System; use System;
with Ada.Unchecked_Deallocation;
with Ada.Unchecked_Conversion;

with Beep_Global; use Beep_Global;
with BeepCom; use BeepCom;

package body Factory is

   ----------------------
   -- Type Conversions --
   ----------------------

   procedure Free is
     new Ada.Unchecked_Deallocation(IClassFactory, IClassFactory_Pointer);
   procedure Free_Vtbl is
     new Ada.Unchecked_Deallocation(IClassFactory_Vtbl_Record,
                                    IClassFactory_Vtbl_Pointer);
   function To_IClassFactory_Pointer is
     new Ada.Unchecked_Conversion (System.Address, IClassFactory_Pointer);
   function To_PVOID is
     new Ada.Unchecked_Conversion (System.Address, Win32.PVOID);

   ----------
   -- Free --
   ----------

   procedure Free (Pointer : in System.Address) is
      X : IClassFactory_Pointer;
   begin
      X := To_IClassFactory_Pointer(Pointer);
      Free(X);
   end Free;

   --------------------
   -- QueryInterface --
   --------------------

   function QueryInterface  (This      : access IClassFactory;
                             riid      : REFIID;
                             PpvObject : access Win32.PVOID)
                             return HRESULT
   is
      lResult : Win32.Long;
   begin

      if riid.all = Beep_Global.IID_IUnknown then
         ppvObject.all := This.all'Address;
      elsif riid.all = Beep_Global.IID_IClassFactory then
         ppvObject.all := This.all'Address;
      else
         ppvObject.all := System.Null_Address;
         return E_NOINTERFACE;
      end if;

      -- When returning new interfaces from any function you must
      -- always add a reference count to that interface directly
      -- or through a call to its AddRef

      lResult := Win32.Winbase.InterlockedIncrement(This.Ref_Count'Access);

      return S_OK;
   end QueryInterface;

   function AddRef (This : access IClassFactory)
                    return Win32.ULONG
   is
      lResult : Win32.Long;
   begin
      -- InterlockedIncrement is a thread protected Win32 API function
      -- increment a long

      lResult := Win32.Winbase.InterlockedIncrement(This.Ref_Count'Access);

      return Win32.ULONG(This.Ref_Count);
   end AddRef;

   function Release (This : access IClassFactory)
                     return Win32.ULONG
   is
   begin
      -- InterlockedDecrement is a thread protected Win32 API function
      -- decrement a long

      if Win32.Winbase.InterlockedDecrement(This.Ref_Count'Access) /=0 then
         -- This.Ref_Count should never be less then 0, so if it is
         -- we get a CONSTRAINT_ERROR exception from Ada
         return Win32.ULONG(This.Ref_Count);
      else
         -- Last reference to IClassFactory and IUnknown released,
         -- so free the object
         Free(This.all'Address);
         return 0;
      end if;

   end Release;

   function CreateInstance (This      : access IClassFactory;
                            pUnkOuter : LPUNKNOWN;
                            riid      : REFIID;
                            ppvObject : access Win32.PVOID)
                            return HRESULT
   is
      function To_IBeep is
        new Ada.Unchecked_Conversion (System.Address, Beep_Vtbl_Pointer);

      Object   : BeepCom.Interface_Pointer;
      hr       : unsigned_long := 16#80040111#;
      lResult  : Win32.LONG;
      refcount : Win32.ULONG;
   begin

      if pUnkOuter /= null then
         return CLASS_E_NOAGGREGATION;
      end if;

      -- Create new COM object

      -- Create an interface for the object
      Object := new BeepCom.Interface;

      -- Create the object and assign in to the interface
      Object.CoClass := new BeepCom.BeepClass;

      -- Ask the object for an interface with IID riid
      hr := To_IBeep(Object.Vtbl).QueryInterface(Object, riid, ppvObject);

      -- When the interface is created with new above it has a default
      -- reference count of 1. If the QueryInterface succeeded it is now 2
      -- if it didn't then the next release will reduce it to 0 and clean
      -- up the Interface (and in this case since there are no other
      -- interfaces referenced clean up the object)
      refcount := To_IBeep(Object.Vtbl).Release(Object);

      lResult := Win32.Winbase.InterlockedIncrement(g_cComponents'Access);

      return hr;
   end CreateInstance;

   function LockServer (
                This  : access IClassFactory;
                FLock : Win32.BOOL)
               return HRESULT is
      use Win32;
      lResult : Win32.LONG;
   begin
      if fLock = Win32.TRUE then
         lResult := Win32.Winbase.InterlockedIncrement(g_cServer'Access);
      else
         lResult := Win32.Winbase.InterlockedDecrement(g_cServer'Access);
      end if;

      -- If this is a local server check to see if server can close
      Can_Close;

      return S_OK;
   end LockServer;

end Factory;

